/* Virginia Tech Cognitive Radio Open Source Systems
 * Virginia Tech, 2009
 *
 * LICENSE INFORMATION GOES HERE
 */

/* This header exports the declarations for all VT-CROSS radio components.  It
 * contains two pure abstract base classes, Component and Engine; Engine derives
 * from Component.  All functions contained within the abstract base classes are
 * dynamically linked and pure, and all child non-abstract classes derive using 
 * private inheritence.  Class functions of the abstract base classes are public
 * for two reasons: (1) To allow for public/protected inheritence in other 
 * implementations, (2) So that symbolic debuggers can navigate the call tree 
 * for typecasted objects of derivative classes.
 */

#ifndef COMPONENTS_H
#define COMPONENTS_H


#include <stdint.h>

#include "containers.h"


class Component
{
    public:
        virtual void GetRemoteComponentType(int32_t socketFD) = 0;
        virtual void WaitForSignal(int32_t socketFD) = 0;
        virtual void Shutdown() = 0;
        virtual void Reset() = 0;
        virtual void RegisterComponent(int32_t socketFD) = 0;
        virtual void DeregisterComponent(int32_t socketFD) = 0;
};


class Engine : public Component
{
    public:
        virtual void RegisterServices(int32_t socketFD) = 0;
        virtual void DeregisterServices(int32_t socketFD) = 0;
};


class ServiceManagementLayer : public Component
{
    public:
        ServiceManagementLayer();
        ~ServiceManagementLayer();

        void GetRemoteComponentType(int32_t socketFD);
        void WaitForSignal(int32_t socketFD);
        void Shutdown();
        void Reset();
        void RegisterComponent(int32_t socketFD);
        void DeregisterComponent(int32_t socketFD);

    private: 
        void TransferRadioConfiguration();
        void TransferExperience();
        void ReceiveServices();
        void SetActiveMission();
        void ListServices();
        void ReloadConfiguration();
        void LoadConfiguration();
};


class PolicyEngine : public Engine
{
    public:
        PolicyEngine();
        ~PolicyEngine();

        void GetRemoteComponentType(int32_t socketFD);
        void WaitForSignal(int32_t socketFD);
        void Shutdown();
        void Reset();
        void RegisterComponent(int32_t socketFD);
        void DeregisterComponent(int32_t socketFD);

        void RegisterServices(int32_t socketFD);
        void DeregisterServices(int32_t socketFD);

    private:
        void LoadPolicies();
        void ReloadPolicies();
        void SendPEDecision(int32_t socketFD, struct Parameter pList[], \
                struct CE_Info *ce_info, int32_t decision_array[]);
        void ValidateParameters(struct Parameter pList[], \
                struct CE_Info *ce_info, int decision_array[]);
};


class CognitiveEngine : public Engine
{
    public:
        CognitiveEngine();
        ~CognitiveEngine();

        void GetRemoteComponentType(int32_t socketFD);
        void WaitForSignal(int32_t socketFD);
        void Shutdown();
        void Reset();
        void RegisterComponent(int32_t socketFD);
        void DeregisterComponent(int32_t socketFD);

        void RegisterServices(int32_t socketFD);
        void DeregisterServices(int32_t socketFD);

    private:
        void ReceiveRadioConfiguration(int32_t socketFD);
        void ReceiveExperience(int32_t socketFD);
        void GetSolution(Observable *observables);
        void GetSolution(Observable *observables, std::string service);
        void ReceiveFeedback(Observable *observables,\
                Parameter *parameters, Utility *utilities);
        void ReceiveFeedback(Observable *observables, \
                Parameter *parameters, std::string service);
};

#endif
